import asyncio
import os
import sys
from dotenv import load_dotenv
from huawei_solar_tools.devices.power_meter import PowerMeter
import huawei_solar_tools.registers.power_meter_register_names as pmrn

async def main():
    # Load environment variables
    load_dotenv("huawei_conf.env")
    host = os.getenv("SMARTLOGGER_IP")
    port = int(os.getenv("SMARTLOGGER_PORT"))
    meter_ids = [int(i) for i in os.getenv("POWER_METER_IDS").split(",")]  

    for device_id in meter_ids:
        # Set up individual report file for each Meter
        script_dir = os.path.dirname(os.path.abspath(__file__))
        reports_dir = os.path.join(script_dir, "power_meter_reports")
        os.makedirs(reports_dir, exist_ok=True)
        report_file = os.path.join(reports_dir, f"power_meter_{device_id}_report.txt")

        with open(report_file, "w") as report:
            sys.stdout = report

            print(f"Connecting to power_meter {device_id}...")

            # Initialize the Inverter
            power_meter = PowerMeter(host, port, device_id)

            try:
                # Test connection
                await power_meter.connect()
                print("Connection successful.")

                # Test connection status
                print(f"Is Power_Meter {device_id} connected? {power_meter.is_connected()}")

                # Test getting device ID
                print(f"power_meter ID: {power_meter.get_device_id()}")

                # Test reading all registers
                results = await power_meter.read_all_registers()
                print("All Registers:")
                for i in results:
                    print(i)

                # Test reading a specific register (grid frequency)
                freq = await power_meter.read_raw_register(32284, 1)
                print(f"Grid Frequency: {freq[0] / 100} Hz")

                # Test reading a specific register by name (Serial Number)
                meter_status = await power_meter.read_register_by_name(pmrn.POWER_FACTOR)
                print(f"Meter Status: {meter_status}")
                
            except Exception as e:
                print(f"Error processing power Meter {device_id}: {e}")

            finally:
                # Test disconnection
                power_meter.disconnect()
                print(f"Disconnected from power Meter {device_id}.")

if __name__ == "__main__":
    asyncio.run(main())
